package com.price2spy.api.rest.client.test;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.StringWriter;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Base64;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.namespace.QName;

import com.price2spy.api.rest.client.exception.ApiAccessNotAllowed;
import com.price2spy.api.rest.client.exception.ApiKeyWrong;
import com.price2spy.api.rest.client.exception.DataValidationError;
import com.price2spy.api.rest.client.exception.GeneralError;
import com.price2spy.api.rest.client.exception.MaxUrlsForPriceplanExceeded;
import com.price2spy.api.rest.client.exception.ProductNameNotUnique;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.price2spy.api.rest.client.bean.BrandType;
import com.price2spy.api.rest.client.bean.GetBrandsResponseType;
import com.price2spy.api.rest.client.bean.GetCurrentPricingDataResponseType;
import com.price2spy.api.rest.client.bean.GetProductsResponseType;
import com.price2spy.api.rest.client.bean.InsertProductType;
import com.price2spy.api.rest.client.bean.InsertUrlType;
import com.price2spy.api.rest.client.bean.ProductType;
import com.price2spy.api.rest.client.bean.ProductsWithUrlsType;
import com.price2spy.api.rest.client.bean.SearchProductsType;
import com.price2spy.api.rest.client.bean.UpdateProductType;
import com.price2spy.api.rest.client.bean.UpdateUrlType;

public class TestClient {

	private static String ENDPOINT_URL = "https://api.price2spy.com/rest";
	private static String API_KEY = "xyz";
	
	
	public static void main(String[] args)  throws GeneralError, ApiAccessNotAllowed, ApiKeyWrong, DataValidationError, ProductNameNotUnique, MaxUrlsForPriceplanExceeded, JAXBException {
		

		List<BrandType> brands = getBrands();
		
		for (BrandType brand: brands) {
			ProductsWithUrlsType productsWithPricing = getCurrentPricingData(brand.getId());
			System.out.println(brand.getName());
			System.out.println("**************");
			System.out.println(toXml(productsWithPricing));
			break;
		}
	
		
		BigInteger productId = insertProduct("Nikon Coolpix L330", "01123456", "Nikon", "Digital camera");
		
		BigInteger url1Id = null;
		BigInteger url2Id = null;
		List<ProductType> products = getProducts("Nikon Coolpix L330");
		for (ProductType product: products) {
			System.out.println(product.getProductId() + ": " + product.getProductName());
			
			url1Id = insertUrl(product.getProductId(), "http://www.amazon.com/Nikon-Coolpix-L330-Digital-Camera/dp/B00HQDBLDO");
			System.out.println("Inserted URL with ID " + url1Id);
			
			url2Id = insertUrl(product.getProductId(), "http://www.target.com/p/nikon-coolpix-l330-20-2mp-digital-camera-with-26x-optical-zoom-black/-/A-16448066");
			System.out.println("Inserted URL with ID " + url2Id);
		}
		
		updateUrl(url1Id.intValue(), "http://www.amazon.com/Nikon-Coolpix-L330-Digital-Camera/dp/B00HQDBLDO", false);
		System.out.println("Updated URL with ID " + url1Id);
		
		updateProduct(productId.intValue(), "Nikon Coolpix L330 updated name", BigDecimal.valueOf(40.55));
		System.out.println("Updated product with ID " + productId.intValue());
		


	}
	
	public static List<BrandType> getBrands() throws GeneralError, ApiAccessNotAllowed, ApiKeyWrong {
		HttpURLConnection connection = null;
	    BufferedReader reader = null;
	    String retVal = null;
	    List<BrandType> brands = null;
	    try {
	      URL restUrl = new URL(ENDPOINT_URL + "/v1/get-brands");
	      connection = (HttpURLConnection) restUrl.openConnection();
	      //Set request method to GET as required from the API
	      connection.setRequestMethod("GET");
	      connection.addRequestProperty("Content-type", "application/json");
	      
	      //Set your API key
	      connection.addRequestProperty("Authorization", getAuth(API_KEY));
	 
	      // Read the response
	      reader = new BufferedReader(new InputStreamReader(connection.getInputStream()));
	      StringBuilder jsonSb = new StringBuilder();
	      String line = null;
	      while ((line = reader.readLine()) != null) {
	        jsonSb.append(line);
	      }
	      retVal = jsonSb.toString();
	      
	      // print out the json response
	      System.out.println(retVal);
	      
	      // Converts JSON string to Java object
	      ObjectMapper mapper = new ObjectMapper();
	      
	      // Converts to an array of Brands
	      GetBrandsResponseType response = mapper.readValue(retVal, GetBrandsResponseType.class);		      
	      brands = response.getBrand();
	 
	    } catch (Exception e) {
	      e.printStackTrace();
	    } finally {
	      // Clean up
	      if (reader != null) {
	        try {
	          reader.close();
	        } catch (IOException e) {
	          e.printStackTrace();
	        }
	      }
	      if (connection != null) {
	        connection.disconnect();
	      }
	    }
		return brands; 
	}

	public static List<ProductType> getProducts(String productName) throws GeneralError, ApiAccessNotAllowed, ApiKeyWrong {

		List<ProductType> products = null;
	    try {
	        URL restUrl = new URL(ENDPOINT_URL + "/v1/get-products");
	        HttpURLConnection con = (HttpURLConnection) restUrl.openConnection();
	        // Set the request method to POST as required from the API
	        con.setRequestMethod("POST");
		    //Set your API key
		    con.addRequestProperty("Authorization", getAuth(API_KEY));
	        // Set the Content-Type to "application/json" as required from the API
	        con.setRequestProperty("Content-Type", "application/json");
	        con.setDoOutput(true);
	        
			SearchProductsType search = new SearchProductsType();
			search.setProductName(productName);
		      
	        OutputStream os = con.getOutputStream();
	        ObjectMapper mapper = new ObjectMapper();
	        os.write(mapper.writeValueAsBytes(search));
	   
	        os.flush();
	        os.close();
	        
	        //get the Products from input stream
	        String productsResponse = "";
	        InputStreamReader in = new InputStreamReader(con.getInputStream());
	        BufferedReader br = new BufferedReader(in);
	        String text = "";
	        while ((text = br.readLine()) != null) {
	        	productsResponse += text;
	        }
	        
		    GetProductsResponseType response = mapper.readValue(productsResponse, GetProductsResponseType.class);		      
		    products = response.getProduct();

	      } catch (Exception e) {
	        e.printStackTrace();
	      }
		return products;	
	}
	
	public static BigInteger insertProduct(String productName, String sku, String brand, String category) throws MaxUrlsForPriceplanExceeded, ApiKeyWrong, DataValidationError, GeneralError, ApiAccessNotAllowed {

		BigInteger response = null;
	    try {
	        URL restUrl = new URL(ENDPOINT_URL + "/v1/insert-product");
	        HttpURLConnection con = (HttpURLConnection) restUrl.openConnection();
	        // Set the request method to POST as required from the API
	        con.setRequestMethod("POST");
		    //Set your API key
		    con.addRequestProperty("Authorization", getAuth(API_KEY));
	        // Set the Content-Type to "application/json" as required from the API
	        con.setRequestProperty("Content-Type", "application/json");
	        con.setDoOutput(true);
	        
			InsertProductType product = new InsertProductType();
			product.setProductName(productName);
			product.setSku(sku);
			product.setBrandName(brand);
			product.setCategoryName(category);
		      
	        OutputStream os = con.getOutputStream();
	        ObjectMapper mapper = new ObjectMapper();
	        os.write(mapper.writeValueAsBytes(product));
	   
	        os.flush();
	        os.close();
	        
	        //get the ProductId from input stream
	        String productId = "";
	        InputStreamReader in = new InputStreamReader(con.getInputStream());
	        BufferedReader br = new BufferedReader(in);
	        String text = "";
	        while ((text = br.readLine()) != null) {
	        	productId += text;
	        }
	        try {
				response = new BigInteger(productId);
			} catch (Exception e) {
				System.out.println(productId);
			}

	        int responseCode = con.getResponseCode();
	        if (responseCode == HttpURLConnection.HTTP_CREATED) {
	          System.out.println("Inserted product successfully.");
	          System.out.println("productId: " + response);
	        } else {
	          System.out.println("Insert product failed." + productId);
	        }
	      } catch (Exception e) {
	        e.printStackTrace();
	      }
		
		return response;
		
	}
	
	public static BigInteger insertUrl(int productId, String url) throws MaxUrlsForPriceplanExceeded, ApiKeyWrong, DataValidationError, GeneralError, ApiAccessNotAllowed {

		
		BigInteger response = null;
	    try {
	        URL restUrl = new URL(ENDPOINT_URL + "/v1/insert-url");
	        HttpURLConnection con = (HttpURLConnection) restUrl.openConnection();
	        // Set the request method to POST as required from the API
	        con.setRequestMethod("POST");
		    //Set your API key
		    con.addRequestProperty("Authorization", getAuth(API_KEY));
	        // Set the Content-Type to "application/json" as required from the API
	        con.setRequestProperty("Content-Type", "application/json");
	        con.setDoOutput(true);
	        
			InsertUrlType insert = new InsertUrlType();
			insert.setProductId(productId);
			insert.setUrl(url);
		      
	        OutputStream os = con.getOutputStream();
	        ObjectMapper mapper = new ObjectMapper();
	        os.write(mapper.writeValueAsBytes(insert));
	   
	        os.flush();
	        os.close();
	        
	        //get the UrlId from input stream
	        String urlId = "";
	        InputStreamReader in = new InputStreamReader(con.getInputStream());
	        BufferedReader br = new BufferedReader(in);
	        String text = "";
	        while ((text = br.readLine()) != null) {
	        	urlId += text;
	        }
	        try {
				response = new BigInteger(urlId);
			} catch (Exception e) {
				System.out.println(productId);
			}

	        int responseCode = con.getResponseCode();
	        if (responseCode == HttpURLConnection.HTTP_CREATED) {
	          System.out.println("Inserted url successfully.");
	          System.out.println("urlId: " + response);
	        } else {
	          System.out.println("Insert url failed. " + urlId);
	        }
	      } catch (Exception e) {
	        e.printStackTrace();
	      }
		
		return response;
	}
	
	public static ProductsWithUrlsType getCurrentPricingData(Integer brandId) throws GeneralError, ApiAccessNotAllowed, ApiKeyWrong {
		
		ProductsWithUrlsType products = null;
	    try {
	        URL restUrl = new URL(ENDPOINT_URL + "/v1/get-current-pricing-data");
	        HttpURLConnection con = (HttpURLConnection) restUrl.openConnection();
	        // Set the request method to POST as required from the API
	        con.setRequestMethod("POST");
		    //Set your API key
		    con.addRequestProperty("Authorization", getAuth(API_KEY));
	        // Set the Content-Type to "application/json" as required from the API
	        con.setRequestProperty("Content-Type", "application/json");
	        con.setDoOutput(true);
	        
			SearchProductsType search = new SearchProductsType();
			search.setBrandId(brandId);
		      
	        OutputStream os = con.getOutputStream();
	        ObjectMapper mapper = new ObjectMapper();
	        
	        os.write(mapper.writeValueAsBytes(search));
	   
	        os.flush();
	        os.close();
	        
	        //get the current pricing data from input stream
	        String pricingData = "";
	        InputStreamReader in = new InputStreamReader(con.getInputStream());
	        BufferedReader br = new BufferedReader(in);
	        String text = "";
	        while ((text = br.readLine()) != null) {
	        	pricingData += text;
	        }	        
	        GetCurrentPricingDataResponseType response = mapper.readValue(pricingData, GetCurrentPricingDataResponseType.class);
	        products = response.getProducts();

	      } catch (Exception e) {
	        e.printStackTrace();
	      }
		return products;
	}
	
	public static void updateUrl(int urlId, String url, boolean active) throws MaxUrlsForPriceplanExceeded, ApiKeyWrong, DataValidationError, GeneralError, ApiAccessNotAllowed {
		
		try {
	        URL restUrl = new URL(ENDPOINT_URL + "/v1/update-url");
	        HttpURLConnection con = (HttpURLConnection) restUrl.openConnection();
	        // Set the request method to POST as required from the API
	        con.setRequestMethod("POST");
		    //Set your API key
		    con.addRequestProperty("Authorization", getAuth(API_KEY));
	        // Set the Content-Type to "application/json" as required from the API
	        con.setRequestProperty("Content-Type", "application/json");
	        con.setDoOutput(true);
	        
			UpdateUrlType product = new UpdateUrlType();
			product.setUrlId(urlId);
			product.setUrl(url);
			product.setActive(active);
 
	        OutputStream os = con.getOutputStream();
	        ObjectMapper mapper = new ObjectMapper();
	        os.write(mapper.writeValueAsBytes(product));
	   
	        os.flush();
	        os.close();

	        int responseCode = con.getResponseCode();
	        if (responseCode == HttpURLConnection.HTTP_OK) {
	          System.out.println("Updated url successfully.");
	        } else {
		       String error = "";
		       InputStreamReader in = new InputStreamReader(con.getInputStream());
		       BufferedReader br = new BufferedReader(in);
		       String text = "";
		       while ((text = br.readLine()) != null) {
		    	   error += text;
		       }
	          System.out.println("Update url failed. " + error);
	        }
	      } catch (Exception e) {
	        e.printStackTrace();
	      }

	}
	
	public static void updateProduct(int productId, String productName, BigDecimal purchasePrice) throws MaxUrlsForPriceplanExceeded, ApiKeyWrong, DataValidationError, GeneralError, ApiAccessNotAllowed {
		
	    try {
	        URL restUrl = new URL(ENDPOINT_URL + "/v1/update-product");
	        HttpURLConnection con = (HttpURLConnection) restUrl.openConnection();
	        // Set the request method to POST as required from the API
	        con.setRequestMethod("POST");
		    //Set your API key
		    con.addRequestProperty("Authorization", getAuth(API_KEY));
	        // Set the Content-Type to "application/json" as required from the API
	        con.setRequestProperty("Content-Type", "application/json");
	        con.setDoOutput(true);
	        
			UpdateProductType product = new UpdateProductType();
			product.setProductId(productId);
			product.setProductName(productName);
			product.setPurchasePrice(purchasePrice);
 
	        OutputStream os = con.getOutputStream();
	        ObjectMapper mapper = new ObjectMapper();
	        os.write(mapper.writeValueAsBytes(product));
	   
	        os.flush();
	        os.close();

	        int responseCode = con.getResponseCode();
	        if (responseCode == HttpURLConnection.HTTP_OK) {
	          System.out.println("Updated product successfully.");
	        } else {
		      String error = "";
		      InputStreamReader in = new InputStreamReader(con.getInputStream());
		      BufferedReader br = new BufferedReader(in);
		      String text = "";
		      while ((text = br.readLine()) != null) {
		    	   error += text;
		      }
	          System.out.println("Update product failed." + error);
	        }
	      } catch (Exception e) {
	        e.printStackTrace();
	      }
	}
	
	public static String getAuth(String apiKey) {
		return "Basic " + Base64.getEncoder().encodeToString(apiKey.getBytes());
	}
	
	public static String toXml(ProductsWithUrlsType products) throws JAXBException {
		JAXBContext jaxbContext = JAXBContext.newInstance(ProductsWithUrlsType.class);
		Marshaller jaxbMarshaller = jaxbContext.createMarshaller();
		jaxbMarshaller.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, true);
		
		StringWriter writer = new StringWriter();
		
		QName qName = new QName("https://api.price2spy.com/rest/test", "products");
        JAXBElement<ProductsWithUrlsType> root = new JAXBElement<ProductsWithUrlsType>(qName, ProductsWithUrlsType.class, products);

        jaxbMarshaller.marshal(root, writer);
        
        return writer.toString();
	}
	
}